//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "LithophaneUnit.h"
#include <jpeg.hpp>
#include "GIFImage.hpp"
#include <IniFiles.hpp>
#include "StringHelp.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TLithophaneForm *LithophaneForm;
//---------------------------------------------------------------------------
__fastcall TLithophaneForm::TLithophaneForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TLithophaneForm::PictureButtonClick(TObject *Sender)
{
    if (OpenDialog1->Execute()) {
        Edit1->Text = OpenDialog1->FileName;
        OpenDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        Edit2->Text = ChangeFileExt(Edit1->Text, ".nc");
        SaveDialog1->InitialDir = ExtractFilePath(Edit1->Text);
        try {
            AnsiString Ext = ExtractFileExt(OpenDialog1->FileName).LowerCase();
            if (Ext == ".bmp" || Ext == ".jpg" || Ext == ".jpeg" || Ext == "*.emf") {
                //this round the houses methodology allows jpegs & emfs to be loaded
                TImage *TempImage = new TImage(this);
                TempImage->AutoSize = true;
                TempImage->Picture->LoadFromFile(OpenDialog1->FileName);
                Picture->Width = TempImage->Width;
                Picture->Height = TempImage->Height;
                Picture->Canvas->Draw(0,0,TempImage->Picture->Graphic);
                delete TempImage;
                }
            if (Ext == ".gif") {
                TGIFImage *Gif = new TGIFImage();
                Gif->LoadFromFile(OpenDialog1->FileName);
                Picture->Assign(Gif);
                delete Gif;
                }
            //Picture->LoadFromFile(Edit1->Text);
            //Label3->Caption = "Picture loaded and is = " + AnsiString(Picture->Width) + "W x " + AnsiString(Picture->Height) + "H Pixels";
            ConversionPossible = true;
            RadioButton1Click(Sender);
            }
        catch (...) {
            Label3->Caption = "Picture did not load";
            ConversionPossible = false;
            }
        }

}
//---------------------------------------------------------------------------
void __fastcall TLithophaneForm::NewGCodeButtonClick(TObject *Sender)
{
    if (SaveDialog1->Execute()) {
        Edit2->Text = SaveDialog1->FileName;
        SaveDialog1->InitialDir = ExtractFilePath(Edit2->Text);
        }

}
//---------------------------------------------------------------------------
void __fastcall TLithophaneForm::FormCreate(TObject *Sender)
{
    ConversionPossible = false;
    Picture = new Graphics::TBitmap();
    //Open Persistant Data & initialise variables
    TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");
    Edit3->Text = IniFile->ReadString("Lithophane","toolD","3.0");
    Edit4->Text = IniFile->ReadString("Lithophane","feedrate","800");
    Edit5->Text = IniFile->ReadString("Lithophane","PicX","0");
    Edit6->Text = IniFile->ReadString("Lithophane","PicY","0");
    Edit7->Text = IniFile->ReadString("Lithophane","WorkZ","0");
    Edit8->Text = IniFile->ReadString("Lithophane","ZeroBrightnessClearance","1");
    Edit9->Text = IniFile->ReadString("Lithophane","Size","100");
    Edit10->Text = IniFile->ReadString("Lithophane","Depth","2.5");
    Edit11->Text = IniFile->ReadString("Lithophane","DepthofCut","2.5");
    RadioButton1->Checked = IniFile->ReadBool("Lithophane","Width",true);
    RadioButton2->Checked = !RadioButton1->Checked;
    CheckBox1->Checked = IniFile->ReadBool("Lithophane","AllowRapidTraverse",true);
    RadioGroup1->OnClick = NULL;
    RadioGroup1->ItemIndex = IniFile->ReadInteger("Lithophane","Units",0);
    RadioGroup1->OnClick = RadioGroup1Click;
    SetUnitLabels((bool) RadioGroup1->ItemIndex);
    useinches = (bool) RadioGroup1->ItemIndex;

    delete IniFile;

}
//---------------------------------------------------------------------------
void __fastcall TLithophaneForm::FormDestroy(TObject *Sender)
{
    delete Picture;
    //Write Persistant Data to the shared cnc .ini file
    TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");
    IniFile->WriteString("Lithophane","toolD",Edit3->Text);
    IniFile->WriteString("Lithophane","feedrate",Edit4->Text);
    IniFile->WriteString("Lithophane","PicX",Edit5->Text);
    IniFile->WriteString("Lithophane","PicY",Edit6->Text);
    IniFile->WriteString("Lithophane","WorkZ",Edit7->Text);
    IniFile->WriteString("Lithophane","ZeroBrightnessClearance",Edit8->Text);
    IniFile->WriteString("Lithophane","Size",Edit9->Text);
    IniFile->WriteString("Lithophane","Depth",Edit10->Text);
    IniFile->WriteString("Lithophane","DepthofCut",Edit11->Text);
    IniFile->WriteBool("Lithophane","Width",RadioButton1->Checked);
    IniFile->WriteBool("Lithophane","AllowRapidTraverse",CheckBox1->Checked);
    IniFile->WriteInteger("Lithophane","Units",RadioGroup1->ItemIndex);

    delete IniFile;

}
//---------------------------------------------------------------------------
void __fastcall TLithophaneForm::HelpButtonClick(TObject *Sender)
{

    AnsiString temp;

    temp = "This program takes a Picture (.bmp, .jpeg, .gif or .emf), and\n";
    temp = temp + "generates a toolpath to machine a lithophane.\n\n";
    temp = temp + "To operate, select picture file name, toolpath filename,\n";
    temp = temp + "fill in all the details, then press the 'Generate' button.\n\n";
    temp = temp + "The toolpath moves to the top left of the picture and runs\n";
    temp = temp + "horizontal scanlines across the picture. Where the picture\n";
    temp = temp + "is black, the tool retracts by the set clearance. For the\n";
    temp = temp + "first scanline the feed is halved as the cutter may be\n";
    temp = temp + "cutting a slot, and there after it speeds up.\n\n";
    temp = temp + "If the 'Depth of Cut' is less than the 'Depth at Max\n";
    temp = temp + "Brightness', then there will be several complete passes\n";
    temp = temp + "over the work, getting deeper with each pass.";
    Application->MessageBox(temp.c_str(),
                            "Help on Lithophane Toolpath Generation Program",
                            MB_OK | MB_ICONINFORMATION);

}
//---------------------------------------------------------------------------

void __fastcall TLithophaneForm::CloseButtonClick(TObject *Sender)
{
   Close();
    
}
//---------------------------------------------------------------------------

void __fastcall TLithophaneForm::RadioButton1Click(TObject *Sender)
{
    //states size of lithophane if it's possible
    if (ConversionPossible) {
        double stepperpixel;
        if (RadioButton1->Checked) {
            //The machined width has been specified
            stepperpixel = (ToDoubleDef(Edit9->Text,100.0) - ToDoubleDef(Edit3->Text,0.0))/((double)Picture->Width);
            } else {
            //The machined height has been specified
            stepperpixel = (ToDoubleDef(Edit9->Text,100.0) - ToDoubleDef(Edit3->Text,0.0))/((double)Picture->Height);
            }
        Label3->Caption = "Lithophane = " + DP(2,stepperpixel * (double)Picture->Width + ToDoubleDef(Edit3->Text,0.0)) + (RadioGroup1->ItemIndex ? "in W x " : "mm W x ") + DP(2,stepperpixel * (double)Picture->Height + ToDoubleDef(Edit3->Text,0.0)) + (RadioGroup1->ItemIndex ? "in" : "mm") + " H. Step per pixel = " + DP(2,stepperpixel) + (RadioGroup1->ItemIndex ? "in." : "mm.");
        }
    
}
//---------------------------------------------------------------------------

void __fastcall TLithophaneForm::GenerateButtonClick(TObject *Sender)
{
    if (!Edit1->Text.Length() || !Edit2->Text.Length()) {
        Application->MessageBox("Need two file names\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (Edit1->Text.LowerCase() == Edit2->Text.LowerCase()) {
        Application->MessageBox("Both file names the same\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (!ConversionPossible) {
        Application->MessageBox("Please Choose a Picture\n\nConversion Aborted.","Warning", MB_OK | MB_ICONERROR);
        Label3->Caption = "Conversion Aborted";
        return;
        }

    if (FileExists(Edit1->Text) && !FileExists(Edit2->Text)) {
        TCursor oldCursor = Cursor;
        try {
            GenerateButton->Enabled = false;
            PictureButton->Enabled = false;
            NewGCodeButton->Enabled = false;
            HelpButton->Enabled = false;
            CloseButton->Enabled = false;
            Edit1->Enabled = false;
            Edit2->Enabled = false;
            Edit3->Enabled = false;
            Edit4->Enabled = false;
            Edit5->Enabled = false;
            Edit6->Enabled = false;
            Edit7->Enabled = false;
            Edit8->Enabled = false;
            Edit9->Enabled = false;
            Edit10->Enabled = false;
            Edit11->Enabled = false;
            RadioGroup1->Enabled = false;
            RadioButton1->Enabled = false;
            RadioButton2->Enabled = false;
            Cursor = crHourGlass;
            Refresh();

            TStringList *GCodeOutputFile = new TStringList();

            //int counter = 0;
            //int filelength = GCodeOutputFile->Count;
            AnsiString temp;
            int maxbrightness = 0;
            int localbrightness;
            int pixelcolour;
            int red;
            int green;
            int blue;
            int NumberofLines;

            //first loop to find out maxbrightness values
            Label3->Caption = "Checking max brightness values";
            Refresh();

            for (int i = 0; i < Picture->Width; i++) {
                for (int j = 0; j < Picture->Height; j++) {
                    pixelcolour = (int) Picture->Canvas->Pixels[i][j];
                    blue = (int) (pixelcolour / 65536);
                    green = (int) ((pixelcolour /256) - blue * 256);
                    red = (int) pixelcolour - green * 256 - blue * 65536;
                    localbrightness = red + green + blue;
                    if (localbrightness > maxbrightness) maxbrightness = localbrightness;
                    }
                }

            if (maxbrightness > 0) {
                Label3->Caption = "Writing toolpath";
                Refresh();

                //second loop does the conversion work

                double currentX;
                double currentY;
                double currentZ;
                double newX;
                double newY;
                double newZ;

                //X & Y steps per pixel
                double stepperpixel;
                if (RadioButton1->Checked) {
                    //The machined width has been specified
                    stepperpixel = (ToDoubleDef(Edit9->Text,100.0) - ToDoubleDef(Edit3->Text,0.0))/((double)Picture->Width);
                    } else {
                    //The machined height has been specified
                    stepperpixel = (ToDoubleDef(Edit9->Text,100.0) - ToDoubleDef(Edit3->Text,0.0))/((double)Picture->Height);
                    }

                //calculate starting position
                double startX = ToDoubleDef(Edit5->Text,0.0) + ToDoubleDef(Edit3->Text,0.0)/2.0;
                double startY = ToDoubleDef(Edit6->Text,0.0) + ToDoubleDef(Edit3->Text,0.0)/2.0 + stepperpixel * (double)Picture->Height ;
                bool firstrow = true;

                //vertical movement for every unitary increase in brightness
                double zstepperbrightness = ToDoubleDef(Edit10->Text,2.5)/((double)maxbrightness);
                if (zstepperbrightness > 0.0) zstepperbrightness = 0.0 - zstepperbrightness;

                //Z for clearnce height over black areas
                double clearanceZ = ToDoubleDef(Edit7->Text,0.0) + ToDoubleDef(Edit8->Text,1.0);

                //Retract plane
                double retractZ = ToDoubleDef(Edit7->Text,0.0) + 10.0 * ToDoubleDef(Edit8->Text,1.0);

                //feedrate
                double feedrate = ToDoubleDef(Edit4->Text,800.0);

                //work height
                double workZ = ToDoubleDef(Edit7->Text,0.0);

                //units
                bool inchUnits = (bool) RadioGroup1->ItemIndex;

                //depth of cut & loop count
                double maxd = ToDoubleDef(Edit10->Text,2.5);
                if (maxd < 0.0) maxd = 0.0 - maxd;
                double doc = ToDoubleDef(Edit11->Text,maxd);
                if (doc > 0.0) doc = 0.0 - doc;
                int loopcount = 0;
                double maxdepthlocal = maxd;
                while (maxdepthlocal > 0.0) {
                    loopcount++;
                    maxdepthlocal += doc;
                    }
                maxd = 0.0 - maxd;
                int brightnessstepspermaxd = (int) (doc/zstepperbrightness);


                //OK, start the lithophane creation.
                //start of file
                GCodeOutputFile->Add("(Lithophane Toolpath File.)");
                GCodeOutputFile->Add("(Toolpath Creation Program  R.W.Williams, 2014)");
                GCodeOutputFile->Add("(Z at top of Work = " + AnsiString(workZ)  + ")");
                GCodeOutputFile->Add("(Cutter Diameter = " + Edit3->Text  + (inchUnits ? " in )" : " mm )"));
                GCodeOutputFile->Add("(Feedrate = " + Edit4->Text  + (inchUnits ? " in/min )" : " mm/min )"));
                GCodeOutputFile->Add("(Picture Bottom Left Corner = X" + Edit5->Text  + " Y" + Edit6->Text + (inchUnits ? " in )" : " mm )"));
                GCodeOutputFile->Add("(Tool Clearance above work for Black areas is " + Edit8->Text  + (inchUnits ? " in )" : " mm )"));
                GCodeOutputFile->Add("(Picture Size = " + AnsiString(Picture->Width) + "W, " + AnsiString(Picture->Height) + "H Pixels)");
                GCodeOutputFile->Add("(Picture Size scanned by cutter= " + DP(4,((double)Picture->Width) * stepperpixel) + "W, " + DP(4,((double)Picture->Height) * stepperpixel) + "H" + (inchUnits ? " in )" : " mm )"));
                GCodeOutputFile->Add("(Picture Size o/a inc. cutter = " + DP(4,((double)Picture->Width) * stepperpixel + ToDoubleDef(Edit3->Text,0.0)) + "W, " + DP(4,((double)Picture->Height) * stepperpixel + ToDoubleDef(Edit3->Text,0.0)) + "H" + (inchUnits ? " in )" : " mm )"));
                GCodeOutputFile->Add("(Step Per Pixel = " + DP(4,stepperpixel) + (inchUnits ? " in )" : " mm )"));
                GCodeOutputFile->Add("(Max Plunge depth = " + Edit10->Text + (inchUnits ? " in )" : " mm )"));
                if (inchUnits) {
                    GCodeOutputFile->Add("(XY Plane, Inch, Absolute distance mode, in/min feedrate)");
                    GCodeOutputFile->Add("G17 G20 G90 G94");
                    } else {
                    GCodeOutputFile->Add("(XY Plane, Metric, Absolute distance mode, mm/min feedrate)");
                    GCodeOutputFile->Add("G17 G21 G90 G94");
                    }
                GCodeOutputFile->Add("(Tool 0, Spindle 1000 rpm. Edit if needed.)");
                GCodeOutputFile->Add("T0 S1000");
                GCodeOutputFile->Add("(Move to retract plane)");
                GCodeOutputFile->Add("G0 Z" + DP(4,retractZ));
                GCodeOutputFile->Add("(Move to start position)");
                GCodeOutputFile->Add("G0 X" + DP(4,startX) + " Y" + DP(4,startY));
                GCodeOutputFile->Add("(Half speed feed for first line as it's being plunged)");
                GCodeOutputFile->Add("F" + DP(2,feedrate/2.0));

                currentX = startX;
                currentY = startY;
                currentZ = retractZ;

                for (int i = 0; i < loopcount; i++) {
                    int pixely = 0;
                    bool traversing = false;
                    while (pixely < Picture->Height) {
                        for (int pixelx = 0; pixelx < Picture->Width; pixelx++) {
                            newX = startX + stepperpixel * ((double) pixelx);
                            newY = startY - stepperpixel * ((double) pixely);
                            pixelcolour = Picture->Canvas->Pixels[pixelx][pixely];
                            blue = (int) (pixelcolour / 65536);
                            green = (int) ((pixelcolour /256) - blue * 256);
                            red = (int) pixelcolour - green * 256 - blue * 65536;
                            localbrightness = red + green + blue;
                            if (localbrightness > (brightnessstepspermaxd * (i + 1))) localbrightness = brightnessstepspermaxd * (i + 1);
                            if (localbrightness == 0) {
                                newZ = clearanceZ;
                                } else {
                                newZ = workZ + zstepperbrightness * ((double)localbrightness);
                                }
                            temp = "G1 ";
                            if (newX != currentX) {
                                if (newY == currentY && newZ == currentZ) {
                                    //it's just traversing
                                    traversing = true;
                                    } else {
                                    if (traversing) {
                                        //the end of a straight traverse has been found
                                        //The last step in the traverse needs to be
                                        //written to the GCode file
                                        //Doesn't speed the GCode up much, just reduces
                                        //the file length.
                                        traversing = false;
                                        //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                        if (currentZ == clearanceZ && CheckBox1->Checked) {
                                            //rapid feed when retracted
                                            GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                            } else {
                                            GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                            }
                                        }
                                    temp = temp + "X" + DP(4,newX) + " ";
                                    }
                                }
                            if (newY != currentY) {
                                if (traversing) {
                                    //the end of a straight traverse has been found
                                    traversing = false;
                                    //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                    if (currentZ == clearanceZ && CheckBox1->Checked) {
                                        //rapid feed when retracted
                                        GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                        } else {
                                        GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                        }
                                    }
                                temp = temp + "Y" + DP(4,newY) + " ";
                                }
                            if (newZ != currentZ) {
                                if (traversing) {
                                    //the end of a straight traverse has been found
                                    traversing = false;
                                    //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                    if (currentZ == clearanceZ && CheckBox1->Checked) {
                                        //rapid feed when retracted
                                        GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                        } else {
                                        GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                        }
                                    }
                                temp = temp + "Z" + DP(4,newZ) + " ";
                                }
                            temp = temp.Trim();
                            if (!traversing && temp != "G1") GCodeOutputFile->Add(temp);
                            currentX = newX;
                            currentY = newY;
                            currentZ = newZ;
                            }
                        if (traversing) {
                            //the end of a straight traverse has been found
                            traversing = false;
                            //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                            if (currentZ == clearanceZ && CheckBox1->Checked) {
                                //rapid feed when retracted
                                GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                } else {
                                GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                }
                            }
                        if (firstrow) {
                            GCodeOutputFile->Add("(Up to full feed speed now the first row is finished)");
                            GCodeOutputFile->Add("F" + DP(2,feedrate));
                            firstrow = false;
                            }
                        pixely++ ;
                        if (pixely < Picture->Height) {
                            for (int pixelx = Picture->Width - 1; pixelx > -1; pixelx--) {
                                newX = startX + stepperpixel * ((double) pixelx);
                                newY = startY - stepperpixel * ((double) pixely);
                                pixelcolour = Picture->Canvas->Pixels[pixelx][pixely];
                                blue = (int) (pixelcolour / 65536);
                                green = (int) ((pixelcolour /256) - blue * 256);
                                red = (int) pixelcolour - green * 256 - blue * 65536;
                                localbrightness = red + green + blue;
                                if (localbrightness > (brightnessstepspermaxd * (i + 1))) localbrightness = brightnessstepspermaxd * (i + 1);
                                if (localbrightness == 0) {
                                    newZ = clearanceZ;
                                    } else {
                                    newZ = workZ + zstepperbrightness * ((double)localbrightness);
                                    }
                                temp = "G1 ";
                                if (newX != currentX) {
                                    if (newY == currentY && newZ == currentZ) {
                                        //it's just traversing
                                        traversing = true;
                                        } else {
                                        if (traversing) {
                                            //the end of a straight traverse has been found
                                            traversing = false;
                                            //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                            if (currentZ == clearanceZ && CheckBox1->Checked) {
                                                //rapid feed when retracted
                                                GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                                } else {
                                                GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                                }
                                            }
                                        temp = temp + "X" + DP(4,newX) + " ";
                                        }
                                    }
                                if (newY != currentY) {
                                    if (traversing) {
                                        //the end of a straight traverse has been found
                                        traversing = false;
                                        //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                        if (currentZ == clearanceZ && CheckBox1->Checked) {
                                            //rapid feed when retracted
                                            GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                            } else {
                                            GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                            }
                                        }
                                    temp = temp + "Y" + DP(4,newY) + " ";
                                    }
                                if (newZ != currentZ) {
                                    if (traversing) {
                                        //the end of a straight traverse has been found
                                        traversing = false;
                                        //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                        if (currentZ == clearanceZ && CheckBox1->Checked) {
                                            //rapid feed when retracted
                                            GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                            } else {
                                            GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                            }
                                        }
                                    temp = temp + "Z" + DP(4,newZ) + " ";
                                    }
                                temp = temp.Trim();
                                if (!traversing && temp != "G1") GCodeOutputFile->Add(temp);
                                currentX = newX;
                                currentY = newY;
                                currentZ = newZ;
                                }
                            }
                        if (traversing) {
                            //the end of a straight traverse has been found
                            traversing = false;
                            //GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                            if (currentZ == clearanceZ && CheckBox1->Checked) {
                                //rapid feed when retracted
                                GCodeOutputFile->Add("G0 X" + DP(4,currentX));
                                } else {
                                GCodeOutputFile->Add("G1 X" + DP(4,currentX));
                                }
                            }

                        pixely++;
                        }
                    if (i < loopcount - 1) {
                        currentX = startX;
                        currentY = startY;
                        currentZ = retractZ;
                        GCodeOutputFile->Add("(Go back to beginning for next pass)");
                        GCodeOutputFile->Add("G1 Z" + DP(4,retractZ));
                        GCodeOutputFile->Add("G0 X" + DP(4,startX) + " Y" + DP(4,startY));
                        }

                    } //end of 'for (int i = 0; i < loopcount; i++)'

                //write G code footer
                GCodeOutputFile->Add("(Retract tool at feed speed and finish)");
                GCodeOutputFile->Add("G1 Z" + DP(4,retractZ));
                GCodeOutputFile->Add("M30");
                NumberofLines = GCodeOutputFile->Count;

                Label3->Caption = "Saving File";
                Refresh();

                GCodeOutputFile->SaveToFile(Edit2->Text);

                } else {
                Application->MessageBox("Picture is all black.\n\nLithophane aborted","Error", MB_OK | MB_ICONERROR);
                }

            Label3->Caption = "Clearing Memory";
            Refresh();

            delete GCodeOutputFile;

            Label3->Caption = "Finished, " + AnsiString(NumberofLines) + " Lines of G Code saved.";
            Refresh();
            }

        __finally {
            GenerateButton->Enabled = true;
            PictureButton->Enabled = true;
            NewGCodeButton->Enabled = true;
            HelpButton->Enabled = true;
            CloseButton->Enabled = true;
            Edit1->Enabled = true;
            Edit2->Enabled = true;
            Edit3->Enabled = true;
            Edit4->Enabled = true;
            Edit5->Enabled = true;
            Edit6->Enabled = true;
            Edit7->Enabled = true;
            Edit8->Enabled = true;
            Edit9->Enabled = true;
            Edit10->Enabled = true;
            Edit11->Enabled = true;
            RadioGroup1->Enabled = true;
            RadioButton1->Enabled = true;
            RadioButton2->Enabled = true;
            Cursor = oldCursor;
            Refresh();
            }

        } else {
        if (FileExists(Edit1->Text)) {
            Application->MessageBox("An existing converted file will get overwritten.\n\nPlease delete file or change converted file name","Warning", MB_OK | MB_ICONERROR);
            } else {
            Application->MessageBox("The picture file does not exist.\n\nPlease check","Warning", MB_OK | MB_ICONERROR);
            }
        Label3->Caption = "Conversion Aborted";
        }

}
//---------------------------------------------------------------------------

void __fastcall TLithophaneForm::SetUnitLabels(bool inches_in_use)
{
    if (!inches_in_use) {
        Label11->Caption = "Tool Diameter (mm)";
        Label4->Caption = "Feedrate (mm/min)";
        Label5->Caption = "Picture Bottom Left X (mm)";
        Label6->Caption = "Picture Bottom Left Y (mm)";
        Label7->Caption = "Work Z (mm)";
        Label8->Caption = "Tool Clearance over Black Areas (mm)";
        Label9->Caption = "Depth at max Brightness (mm)";
        Label12->Caption = "Depth of Cut (mm)";
        Label10->Caption = "Dimension (mm)";
        } else {
        Label11->Caption = "Tool Diameter (in)";
        Label4->Caption = "Feedrate (in/min)";
        Label5->Caption = "Picture Bottom Left X (in)";
        Label6->Caption = "Picture Bottom Left Y (in)";
        Label7->Caption = "Work Z (in)";
        Label8->Caption = "Tool Clearance over Black Areas (in)";
        Label9->Caption = "Depth at max Brightness (in)";
        Label12->Caption = "Depth of Cut (in)";
        Label10->Caption = "Dimension (in)";
        }
}
//---------------------------------------------------------------------------

void __fastcall TLithophaneForm::RadioGroup1Click(TObject *Sender)
{
    if ((bool) RadioGroup1->ItemIndex != useinches) {
        //the click has changed the units
        if (useinches) {
            //going to use mm now
            Edit3->Text = AnsiString(DP(4,ToDoubleDef(Edit3->Text,3.0) * 25.4));
            Edit4->Text = AnsiString(DP(4,ToDoubleDef(Edit4->Text,3.0) * 25.4));
            Edit5->Text = AnsiString(DP(4,ToDoubleDef(Edit5->Text,3.0) * 25.4));
            Edit6->Text = AnsiString(DP(4,ToDoubleDef(Edit6->Text,3.0) * 25.4));
            Edit7->Text = AnsiString(DP(4,ToDoubleDef(Edit7->Text,3.0) * 25.4));
            Edit8->Text = AnsiString(DP(4,ToDoubleDef(Edit8->Text,3.0) * 25.4));
            Edit9->Text = AnsiString(DP(4,ToDoubleDef(Edit9->Text,3.0) * 25.4));
            Edit10->Text = AnsiString(DP(4,ToDoubleDef(Edit10->Text,3.0) * 25.4));
            Edit11->Text = AnsiString(DP(4,ToDoubleDef(Edit11->Text,3.0) * 25.4));
            } else {
            //going to use inches now
            Edit3->Text = AnsiString(DP(4,ToDoubleDef(Edit3->Text,3.0) / 25.4));
            Edit4->Text = AnsiString(DP(4,ToDoubleDef(Edit4->Text,3.0) / 25.4));
            Edit5->Text = AnsiString(DP(4,ToDoubleDef(Edit5->Text,3.0) / 25.4));
            Edit6->Text = AnsiString(DP(4,ToDoubleDef(Edit6->Text,3.0) / 25.4));
            Edit7->Text = AnsiString(DP(4,ToDoubleDef(Edit7->Text,3.0) / 25.4));
            Edit8->Text = AnsiString(DP(4,ToDoubleDef(Edit8->Text,3.0) / 25.4));
            Edit9->Text = AnsiString(DP(4,ToDoubleDef(Edit9->Text,3.0) / 25.4));
            Edit10->Text = AnsiString(DP(4,ToDoubleDef(Edit10->Text,3.0) / 25.4));
            Edit11->Text = AnsiString(DP(4,ToDoubleDef(Edit11->Text,3.0) / 25.4));
            }
        useinches = (bool) RadioGroup1->ItemIndex;
        }
    SetUnitLabels(useinches);
}
//---------------------------------------------------------------------------

